#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          09-07-2018
//      Created for Project :   FLAME
"""

import pytest
from influxdb import InfluxDBClient
from clmcservice.generate_network_measurements import report_network_measurements
from py2neo import Graph

network_config = {
    "links": [
        {
            "source": "DC1",
            "target": "DC2",
            "min_response_time": 10,
            "max_response_time": 20,
            "avg_response_time": 15
        },
        {
            "source": "DC2",
            "target": "DC1",
            "min_response_time": 16,
            "max_response_time": 28,
            "avg_response_time": 22
        },
        {
            "source": "DC1",
            "target": "DC3",
            "min_response_time": 17,
            "max_response_time": 19,
            "avg_response_time": 18
        },
        {
            "source": "DC3",
            "target": "DC1",
            "min_response_time": 15,
            "max_response_time": 25,
            "avg_response_time": 20
        },
        {
            "source": "DC1",
            "target": "DC5",
            "min_response_time": 27,
            "max_response_time": 33,
            "avg_response_time": 30
        },
        {
            "source": "DC5",
            "target": "DC1",
            "min_response_time": 10,
            "max_response_time": 42,
            "avg_response_time": 26
        },
        {
            "source": "DC2",
            "target": "DC4",
            "min_response_time": 11,
            "max_response_time": 29,
            "avg_response_time": 20
        },
        {
            "source": "DC4",
            "target": "DC2",
            "min_response_time": 12,
            "max_response_time": 40,
            "avg_response_time": 26
        },
        {
            "source": "DC3",
            "target": "DC4",
            "min_response_time": 23,
            "max_response_time": 27,
            "avg_response_time": 25
        },
        {
            "source": "DC4",
            "target": "DC3",
            "min_response_time": 12,
            "max_response_time": 18,
            "avg_response_time": 15
        },
        {
            "source": "DC5",
            "target": "DC6",
            "min_response_time": 3,
            "max_response_time": 15,
            "avg_response_time": 9
        },
        {
            "source": "DC6",
            "target": "DC5",
            "min_response_time": 11,
            "max_response_time": 11,
            "avg_response_time": 11
        },
    ]
}


@pytest.fixture(scope='module', autouse=True)
def db_testing_data():
    """
    This fixture generates some testing data in influx to be used for testing, after which it clears up the DB.

    :return: a pair of time stamps defining the from-to range for which the test data is reported
    """

    global network_config

    test_db_name = "TestInfluxDB"

    influx = InfluxDBClient(host="localhost", port=8086, timeout=10)
    graph = Graph(host="localhost", password="admin")
    graph.delete_all()

    dbs = influx.get_list_database()
    if "CLMCMetrics" not in dbs:
        influx.create_database("CLMCMetrics")
    report_network_measurements("localhost", "CLMCMetrics", network_config, "localhost", "admin")

    if test_db_name in dbs:
        influx.drop_database(test_db_name)
    influx.create_database(test_db_name)
    influx.switch_database(test_db_name)

    from_timestamp = 1528385860
    to_timestamp = 1528685860

    data = [
        ("host1", "nginx_1_ep1", "DC4", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr4", 5, 20, 1528385860),
        ("host2", "nginx_1_ep2", "DC6", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr6", 8, 35, 1528385860),
        ("host1", "nginx_1_ep1", "DC4", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr4", 7, 15, 1528389860),
        ("host2", "nginx_1_ep2", "DC6", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr6", 10, 23, 1528389860),
        ("host1", "nginx_1_ep1", "DC4", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr4", 12, 17, 1528395860),
        ("host2", "nginx_1_ep2", "DC6", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr6", 15, 11, 1528395860),
        ("host1", "nginx_1_ep1", "DC4", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr4", 17, 23, 1528485860),
        ("host2", "nginx_1_ep2", "DC6", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr6", 19, 24, 1528485860),
        ("host1", "nginx_1_ep1", "DC4", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr4", 11, 16, 1528545860),
        ("host2", "nginx_1_ep2", "DC6", "nginx", "nginx_1", "test_sfc1", "test_sfc1_1", "sr6", 20, 18, 1528545860)
    ]
    influx.write_points([
        {"measurement": "nginx",
         "tags": {"host": host, "ipendpoint": endpoint, "location": location, "sf": sf, "sf_i": sf_i, "sfc": sfc, "sfc_i": sfc_i, "sr": sr},
         "fields": {"requests": num_requests, "avg_processing_time": processing_time},
         "time": timestamp * 10 ** 9
         } for host, endpoint, location, sf, sf_i, sfc, sfc_i, sr, num_requests, processing_time, timestamp in data
    ])

    data = [
        ("host3", "minio_1_ep1", "DC4", "minio", "minio_1", "test_sfc1", "test_sfc1_1", "sr4", 12, 86, 1528386860),
        ("host4", "minio_2_ep1", "DC5", "minio", "minio_2", "test_sfc2", "test_sfc2_1", "sr5", 15, 75, 1528386860),
        ("host3", "minio_1_ep1", "DC4", "minio", "minio_1", "test_sfc1", "test_sfc1_1", "sr4", 7, 105, 1528388860),
        ("host4", "minio_2_ep1", "DC5", "minio", "minio_2", "test_sfc2", "test_sfc2_1", "sr5", 12, 60, 1528388860),
        ("host3", "minio_1_ep1", "DC4", "minio", "minio_1", "test_sfc1", "test_sfc1_1", "sr4", 11, 121, 1528410860),
        ("host4", "minio_2_ep1", "DC5", "minio", "minio_2", "test_sfc2", "test_sfc2_1", "sr5", 12, 154, 1528410860),
        ("host3", "minio_1_ep1", "DC4", "minio", "minio_1", "test_sfc1", "test_sfc1_1", "sr4", 14, 84, 1528412860),
        ("host4", "minio_2_ep1", "DC5", "minio", "minio_2", "test_sfc2", "test_sfc2_1", "sr5", 5, 45, 1528412860),
        ("host3", "minio_1_ep1", "DC4", "minio", "minio_1", "test_sfc1", "test_sfc1_1", "sr4", 7, 63, 1528414860),
        ("host4", "minio_2_ep1", "DC5", "minio", "minio_2", "test_sfc2", "test_sfc2_1", "sr5", 16, 86, 1528414860)
    ]
    influx.write_points([
        {"measurement": "minio_http",
         "tags": {"host": host, "ipendpoint": endpoint, "location": location, "sf": sf, "sf_i": sf_i, "sfc": sfc, "sfc_i": sfc_i, "sr": sr},
         "fields": {"total_requests_count": num_requests, "total_processing_time": processing_time},
         "time": timestamp * 10 ** 9
         } for host, endpoint, location, sf, sf_i, sfc, sfc_i, sr, num_requests, processing_time, timestamp in data
    ])

    data = [
        ("host5", "apache_1_ep1", "DC5", "apache", "apache_1", "test_sfc2", "test_sfc2_1", "sr5", 15, 1528386860),
        ("host5", "apache_1_ep1", "DC5", "apache", "apache_1", "test_sfc2", "test_sfc2_1", "sr5", 17, 1528388860),
        ("host5", "apache_1_ep1", "DC5", "apache", "apache_1", "test_sfc2", "test_sfc2_1", "sr5", 19, 1528410860),
        ("host5", "apache_1_ep1", "DC5", "apache", "apache_1", "test_sfc2", "test_sfc2_1", "sr5", 24, 1528412860),
        ("host5", "apache_1_ep1", "DC5", "apache", "apache_1", "test_sfc2", "test_sfc2_1", "sr5", 13, 1528414860),
    ]
    influx.write_points([
        {"measurement": "apache",
         "tags": {"host": host, "ipendpoint": endpoint, "location": location, "sf": sf, "sf_i": sf_i, "sfc": sfc, "sfc_i": sfc_i, "sr": sr},
         "fields": {"avg_processing_time": processing_time},
         "time": timestamp * 10 ** 9
         } for host, endpoint, location, sf, sf_i, sfc, sfc_i, sr, processing_time, timestamp in data
    ])

    yield from_timestamp, to_timestamp, test_db_name, graph

    influx.drop_database("CLMCMetrics")
    influx.drop_database("TestInfluxDB")
    graph.delete_all()
